use crate::common::Waitable;
use crate::{Channel, TallyState};
use crossbeam_channel::{Receiver, Sender};
use rand::prelude::*;
use std::thread;
use std::thread::JoinHandle;
use std::time::Duration;

/// An Adaptor interfaces to a system providing tally information, e.g. a BlackMagic ATEM
pub trait Adaptor: Waitable {
    /// Run the adaptor, returning a channel receiver which receives TallyStates
    fn run(&mut self) -> Receiver<TallyState>;
}

pub struct FakeAdaptor {
    numchannels: u8,
    thread: Option<thread::JoinHandle<()>>,
}

impl FakeAdaptor {
    pub fn new(numchannels: u8) -> FakeAdaptor {
        return FakeAdaptor {
            numchannels: numchannels,
            thread: None,
        };
    }

    fn _run(numchannels: u8, sender: Sender<TallyState>) {
        loop {
            let state = TallyState {
                on_air: Channel(Some(rand::thread_rng().gen_range(0, numchannels))),
                preview: Channel(Some(rand::thread_rng().gen_range(0, numchannels))),
            };
            if let Err(_) = sender.send(state) {
                log::info!("Fakeadaptor shutting down");
                break;
            };
            thread::sleep(Duration::new(rand::thread_rng().gen_range(0, 5), 0));
        }
    }
}

impl Waitable for FakeAdaptor {
    fn get_joinhandle(self) -> Option<JoinHandle<()>> {
        self.thread
    }
}

impl Adaptor for FakeAdaptor {
    fn run(&mut self) -> Receiver<TallyState> {
        log::info!("Fakeadaptor starting");
        let (tx, rx) = crossbeam_channel::unbounded::<TallyState>();
        let thread_numchannels = self.numchannels.clone();
        self.thread = Some(thread::spawn(move || {
            FakeAdaptor::_run(thread_numchannels, tx);
        }));
        return rx;
    }
}
