mod adaptors;
mod common;
mod drivers;
mod web;
use crate::adaptors::{Adaptor, FakeAdaptor};
use crate::drivers::{Driver, FakeDriver};
use crossbeam_channel::{select, Receiver};
use simplelog::{Config, LevelFilter, TermLogger};
use std::fmt;
use web::WebServer;

#[derive(Clone)]
pub struct Channel(Option<u8>);

#[derive(Clone)]
pub struct TallyState {
    on_air: Channel,
    preview: Channel,
}

impl fmt::Display for Channel {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match self.0 {
            Some(n) => write!(f, "{}", n),
            None => write!(f, "-"),
        }
    }
}

impl fmt::Display for TallyState {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "On Air: {}, Preview: {}", self.on_air, self.preview)
    }
}

struct Server<T: Adaptor, U: Driver> {
    adaptor: T,
    driver: U,
}

impl<T: Adaptor, U: Driver> Server<T, U> {
    fn run(mut self, signal_rx: Receiver<()>) {
        let rx = self.adaptor.run();
        let tx = self.driver.run();
        let webaddr = ([127, 0, 0, 1], 3000).into();
        //let (websocket_tx, websocket_rx) =
        log::info!("Webserver starting on: {}", webaddr);
        let web = WebServer::run(webaddr);
        loop {
            select! {
                recv(signal_rx) -> _ => {
                    log::info!("Signal received, quitting!");
                    // signal webserver to shutdown and wait for exit..
                    web.shutdown();
                    // Drop the driver channel, and wait for it to shutdown...
                    drop(tx);
                    self.driver.wait();

                    // ...and do the same for the adaptor.
                    drop(rx);
                    self.adaptor.wait();

                    // We're done, break loop and return.
                    break;
                },
                recv(rx) -> state => {
                    tx.send(state.unwrap()).unwrap();
                }
            }
        }
    }
}

fn main() {
    TermLogger::init(LevelFilter::Debug, Config::default()).unwrap();

    let (signal_tx, signal_rx) = crossbeam_channel::bounded::<()>(1);
    ctrlc::set_handler(move || {
        log::info!("Received interrupt. Shutting down...");
        signal_tx.send(()).expect("Error sending to signal channel");
    })
    .expect("Error setting signal handler");

    let s = Server {
        adaptor: FakeAdaptor::new(8),
        driver: FakeDriver::new(),
    };

    s.run(signal_rx);
}
